@@ -1,5 +1,5 @@
 package Net::Hadoop::Oozie;
-$Net::Hadoop::Oozie::VERSION = '0.114';
+$Net::Hadoop::Oozie::VERSION = '0.115';
 use 5.010;
 use strict;
 use warnings;
@@ -974,6 +974,96 @@ sub _jobs_iterator {
     return;
 }
 
+sub failed_last_n_hours{
+    my ( $self, $workflow, $n_hours ) = @_;
+    return (
+                (
+                    !$workflow->{endTime_epoch}
+                    && $workflow->{startTime_epoch} >= time - $n_hours * 3600
+                )
+                ||
+                (
+                    $workflow->{endTime_epoch}
+                    && $workflow->{endTime_epoch} >= time - $n_hours * 3600
+                )
+            );
+}
+
+sub failed_workflows_nr {
+    my ( $self, $n_hours ) = @_;
+    $self->filter( { status => [qw(FAILED SUSPENDED KILLED)] } );
+    my $jobs = $self->jobs( jobtype => 'workflows', len => 1_000 );
+    return scalar( grep { $self->failed_last_n_hours( $_, $n_hours ) } @{$jobs->{workflows}} );
+}
+
+sub failed_workflows_last_n_hours_paged {
+    my $self    = shift;
+    my $n_hours = shift || 1;
+    my $pattern = shift;
+    my $opt     = shift || { parent_info => 1 };
+
+    my $want_parent_info   = $opt->{parent_info};
+    my $page_size          = $opt->{page_size} // 50;
+    my $page_nr            = $opt->{page_nr}   //  1;
+    my $current_pos        = 1;
+    my $failed_workflow_nr = $self->failed_workflows_nr( $n_hours );
+    my $total_page_nr      = ceil( $failed_workflow_nr/$page_size );
+
+    $page_nr = $total_page_nr if $page_nr>$total_page_nr;
+
+    my(@failed, $console_url_base);
+    my $cb = sub {
+        my $jobs = shift;
+        my $workflows = $jobs->{workflows};
+        for my $workflow ( @$workflows ){
+            next if ($pattern && $workflow->{appName} !~ /$pattern/);
+            if ( $self->failed_last_n_hours($workflow,$n_hours) )
+                {
+                    if( $current_pos <= $page_size * ($page_nr-1) ){
+                        $current_pos ++;
+                        next;
+                    }
+                    if ( !$console_url_base ) {
+                        ( $console_url_base = $workflow->{consoleUrl} ) =~ s/job=.*/job=/;
+                    }
+                    my $details =  $self->job( $workflow->{id} );
+                    my ($error) = map { $_->{errorMessage} ? $_->{errorMessage} : () } @{$details->{actions}||[]};
+                    my $conf = eval { xml_in($details->{conf}) } || {};
+                    for (qw(timeoutSkipErrorMail errorEmailTo)) {
+                        $workflow->{$_} = $conf->{property}{$_}{value};
+                    }
+                    my $parent_id = $workflow->{parentId} = $details->{parentId}
+                                    // "";
+                    if ($parent_id && $want_parent_info ) {
+                        $parent_id =~ s/\@[0-9]+$//;
+                        my $parent = $self->job($parent_id);
+                        $workflow->{parentConsoleUrl}
+                            = $parent->{coordJobId}
+                            ? $console_url_base . $parent->{coordJobId}
+                            : 'not found';
+                        $workflow->{parentStatus}  = $parent->{status};
+                        $workflow->{parentAppname} = $parent->{coordJobName};
+                        $workflow->{parentId}      = $parent->{coordJobId};
+                        $workflow->{scheduled}++;
+                    }
+                    $workflow->{errorMessage}  = $error || '-';
+                    push @failed, $workflow;
+                    if (
+                        @failed >= $page_size
+                        || @failed >= $failed_workflow_nr - $page_size*($page_nr-1)
+                    ) {
+                        return;
+                    }
+                }
+        }
+        return 1;
+    };
+
+    $self->_jobs_iterator({ is_coordinator => 0, callback => $cb });
+
+    return ( $total_page_nr, $page_nr, \@failed );
+}
+
 1;
 
 __END__
@@ -988,7 +1078,7 @@ Net::Hadoop::Oozie - Interface to various Oozie REST endpoints and utility metho
 
 =head1 VERSION
 
-version 0.114
+version 0.115
 
 =head1 DESCRIPTION
 
@@ -1203,8 +1293,34 @@ Having coordinators like this is usually an user error when submitting jobs.
 
 =head3 failed_workflows_last_n_hours
 
+    my %options = ( # all keys are optional
+        parent_info => Bool, # default: 1
+    );
+
+    my $failed_arrayref = $oozie->failed_workflows_last_n_hours( $hours, $pattern, \%options );
+
 =head3 failed_workflows_last_n_hours_pretty
 
+    my $string = $oozie->failed_workflows_last_n_hours_pretty( $hours );
+
+=head3 failed_workflows_last_n_hours_paged
+
+    my %options = ( # all keys are optional
+        parent_info => Bool, # default: 1
+        page_size   => Int,  # default: 50
+        page_nr     => Int,  # default: 1
+    );
+
+    my(
+        $total_page_nr,
+        $page_nr,
+        $failed_arrayref,
+    ) = $oozie->failed_workflows_last_n_hours_paged(
+            $hours,
+            $pattern,
+            \%options,
+        );
+
 =head3 job_exists
 
 This is a sugar interface on top of the L</job> method. Normally the REST interface
